@@ -23,15 +23,19 @@
 #include <QTimer>
 #include <QtConcurrent>
 
+#include "loggingcategory.h"
 #include <cmath>
+#include <vector>
 
-#define return_fallback(value) if (m_imageData.m_samples.size() == 0) {\
-    return value;\
-}
+#define return_fallback(value)                                                                                                                                 \
+    if (m_imageData.m_samples.size() == 0) {                                                                                                                   \
+        return value;                                                                                                                                          \
+    }
 
-#define return_fallback_finally(value, finally) if (m_imageData.m_samples.size() == 0) {\
-    return value.isValid() ? value : static_cast<Kirigami::PlatformTheme*>(qmlAttachedPropertiesObject<Kirigami::PlatformTheme>(this, true))->finally();\
-}
+#define return_fallback_finally(value, finally)                                                                                                                \
+    if (m_imageData.m_samples.size() == 0) {                                                                                                                   \
+        return value.isValid() ? value : static_cast<Kirigami::PlatformTheme *>(qmlAttachedPropertiesObject<Kirigami::PlatformTheme>(this, true))->finally();  \
+    }
 
 ImageColors::ImageColors(QObject *parent)
     : QObject(parent)
@@ -39,13 +43,14 @@ ImageColors::ImageColors(QObject *parent)
     m_imageSyncTimer = new QTimer(this);
     m_imageSyncTimer->setSingleShot(true);
     m_imageSyncTimer->setInterval(100);
-   /* connect(m_imageSyncTimer, &QTimer::timeout, this, [this]() {
-       generatePalette();
-    });*/
+    /* connect(m_imageSyncTimer, &QTimer::timeout, this, [this]() {
+        generatePalette();
+     });*/
 }
 
 ImageColors::~ImageColors()
-{}
+{
+}
 
 void ImageColors::setSource(const QVariant &source)
 {
@@ -62,7 +67,7 @@ void ImageColors::setSource(const QVariant &source)
     }
 
     m_source = source;
-    emit sourceChanged();
+    Q_EMIT sourceChanged();
 }
 
 QVariant ImageColors::source() const
@@ -110,19 +115,17 @@ void ImageColors::setSourceItem(QQuickItem *source)
     update();
 
     if (m_sourceItem) {
-        auto syncWindow = [this] () {
+        auto syncWindow = [this]() {
             if (m_window) {
                 disconnect(m_window.data(), nullptr, this, nullptr);
             }
             m_window = m_sourceItem->window();
             if (m_window) {
-                connect(m_window, &QWindow::visibleChanged,
-                        this, &ImageColors::update);
+                connect(m_window, &QWindow::visibleChanged, this, &ImageColors::update);
             }
         };
 
-        connect(m_sourceItem, &QQuickItem::windowChanged,
-                this, syncWindow);
+        connect(m_sourceItem, &QQuickItem::windowChanged, this, syncWindow);
         syncWindow();
     }
 }
@@ -139,19 +142,20 @@ void ImageColors::update()
         m_futureImageData->deleteLater();
     }
     auto runUpdate = [this]() {
-        QFuture<ImageData> future = QtConcurrent::run([this](){return generatePalette(m_sourceImage);});
+        QFuture<ImageData> future = QtConcurrent::run([this]() {
+            return generatePalette(m_sourceImage);
+        });
         m_futureImageData = new QFutureWatcher<ImageData>(this);
-        connect(m_futureImageData, &QFutureWatcher<ImageData>::finished,
-                this, [this] () {
-                    if (!m_futureImageData) {
-                        return;
-                    }
-                    m_imageData = m_futureImageData->future().result();
-                    m_futureImageData->deleteLater();
-                    m_futureImageData = nullptr;
+        connect(m_futureImageData, &QFutureWatcher<ImageData>::finished, this, [this]() {
+            if (!m_futureImageData) {
+                return;
+            }
+            m_imageData = m_futureImageData->future().result();
+            m_futureImageData->deleteLater();
+            m_futureImageData = nullptr;
 
-                    emit paletteChanged();
-                });
+            Q_EMIT paletteChanged();
+        });
         m_futureImageData->setFuture(future);
     };
 
@@ -183,13 +187,13 @@ inline int squareDistance(QRgb color1, QRgb color2)
     // https://en.wikipedia.org/wiki/Color_difference
     // Using RGB distance for performance, as CIEDE2000 istoo complicated
     if (qRed(color1) - qRed(color2) < 128) {
-        return 2 * pow(qRed(color1) - qRed(color2), 2) +
-            4 * pow(qGreen(color1) - qGreen(color2), 2) +
-            3 * pow(qBlue(color1) - qBlue(color2), 2);
+        return 2 * pow(qRed(color1) - qRed(color2), 2) //
+            + 4 * pow(qGreen(color1) - qGreen(color2), 2) //
+            + 3 * pow(qBlue(color1) - qBlue(color2), 2);
     } else {
-        return 3 * pow(qRed(color1) - qRed(color2), 2) +
-            4 * pow(qGreen(color1) - qGreen(color2), 2) +
-            2 * pow(qBlue(color1) - qBlue(color2), 2);
+        return 3 * pow(qRed(color1) - qRed(color2), 2) //
+            + 4 * pow(qGreen(color1) - qGreen(color2), 2) //
+            + 2 * pow(qBlue(color1) - qBlue(color2), 2);
     }
 }
 
@@ -244,7 +248,7 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage)
         return imageData;
     }
 
-    imageData.m_average = QColor(r/c, g/c, b/c, 255);
+    imageData.m_average = QColor(r / c, g / c, b / c, 255);
 
     for (int iteration = 0; iteration < 5; ++iteration) {
         for (auto &stat : imageData.m_clusters) {
@@ -253,7 +257,7 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage)
             b = 0;
             c = 0;
 
-            for (auto color : qAsConst(stat.colors)) {
+            for (auto color : std::as_const(stat.colors)) {
                 c++;
                 r += qRed(color);
                 g += qGreen(color);
@@ -267,38 +271,36 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage)
             stat.colors = QList<QRgb>({stat.centroid});
         }
 
-        for (auto color : qAsConst(imageData.m_samples)) {
+        for (auto color : std::as_const(imageData.m_samples)) {
             positionColor(color, imageData.m_clusters);
         }
     }
 
     std::sort(imageData.m_clusters.begin(), imageData.m_clusters.end(), [](const ImageData::colorStat &a, const ImageData::colorStat &b) {
-        return a.colors.size() > b.colors.size();   
+        return a.colors.size() > b.colors.size();
     });
 
     // compress blocks that became too similar
     auto sourceIt = imageData.m_clusters.end();
-    QList<QList<ImageData::colorStat>::iterator> itemsToDelete;
+    // Use index instead of iterator, because QList::erase may invalidate iterator.
+    std::vector<int> itemsToDelete;
     while (sourceIt != imageData.m_clusters.begin()) {
         sourceIt--;
         for (auto destIt = imageData.m_clusters.begin(); destIt != imageData.m_clusters.end() && destIt != sourceIt; destIt++) {
             if (squareDistance((*sourceIt).centroid, (*destIt).centroid) < s_minimumSquareDistance) {
                 const qreal ratio = (*sourceIt).ratio / (*destIt).ratio;
-                const int r = ratio * qreal(qRed((*sourceIt).centroid)) +
-                    (1 - ratio) * qreal(qRed((*destIt).centroid));
-                const int g = ratio * qreal(qGreen((*sourceIt).centroid)) +
-                    (1 - ratio) * qreal(qGreen((*destIt).centroid));
-                const int b = ratio * qreal(qBlue((*sourceIt).centroid)) +
-                    (1 - ratio) * qreal(qBlue((*destIt).centroid));
+                const int r = ratio * qreal(qRed((*sourceIt).centroid)) + (1 - ratio) * qreal(qRed((*destIt).centroid));
+                const int g = ratio * qreal(qGreen((*sourceIt).centroid)) + (1 - ratio) * qreal(qGreen((*destIt).centroid));
+                const int b = ratio * qreal(qBlue((*sourceIt).centroid)) + (1 - ratio) * qreal(qBlue((*destIt).centroid));
                 (*destIt).ratio += (*sourceIt).ratio;
                 (*destIt).centroid = qRgb(r, g, b);
-                itemsToDelete << sourceIt;
+                itemsToDelete.push_back(std::distance(imageData.m_clusters.begin(), sourceIt));
                 break;
             }
         }
     }
-    for (const auto &i : qAsConst(itemsToDelete)) {
-        imageData.m_clusters.erase(i);
+    for (auto i : std::as_const(itemsToDelete)) {
+        imageData.m_clusters.removeAt(i);
     }
 
     imageData.m_highlight = QColor();
@@ -310,19 +312,19 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage)
 
     bool first = true;
 
-    for (const auto &stat : qAsConst(imageData.m_clusters)) {
+    for (const auto &stat : std::as_const(imageData.m_clusters)) {
         QVariantMap entry;
         const QColor color(stat.centroid);
         entry[QStringLiteral("color")] = color;
         entry[QStringLiteral("ratio")] = stat.ratio;
 
         QColor contrast = QColor(255 - color.red(), 255 - color.green(), 255 - color.blue());
-        contrast.setHsl(contrast.hslHue(),
-                        contrast.hslSaturation(),
+        contrast.setHsl(contrast.hslHue(), //
+                        contrast.hslSaturation(), //
                         128 + (128 - contrast.lightness()));
         QColor tempContrast;
-        int minimumDistance = 4681800; //max distance: 4*3*2*3*255*255
-        for (const auto &stat : qAsConst(imageData.m_clusters)) {
+        int minimumDistance = 4681800; // max distance: 4*3*2*3*255*255
+        for (const auto &stat : std::as_const(imageData.m_clusters)) {
             const int distance = squareDistance(contrast.rgb(), stat.centroid);
 
             if (distance < minimumDistance) {
@@ -331,23 +333,20 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage)
             }
         }
 
-
         if (imageData.m_clusters.size() <= 3) {
             if (qGray(imageData.m_dominant.rgb()) < 120) {
                 contrast = QColor(230, 230, 230);
             } else {
                 contrast = QColor(20, 20, 20);
             }
-        // TODO: replace m_clusters.size() > 3 with entropy calculation
+            // TODO: replace m_clusters.size() > 3 with entropy calculation
         } else if (squareDistance(contrast.rgb(), tempContrast.rgb()) < s_minimumSquareDistance * 1.5) {
             contrast = tempContrast;
         } else {
             contrast = tempContrast;
             contrast.setHsl(contrast.hslHue(),
                             contrast.hslSaturation(),
-                            contrast.lightness() > 128
-                                ? qMin(contrast.lightness() + 20, 255)
-                                : qMax(0, contrast.lightness() - 20));
+                            contrast.lightness() > 128 ? qMin(contrast.lightness() + 20, 255) : qMax(0, contrast.lightness() - 20));
         }
 
         entry[QStringLiteral("contrastColor")] = contrast;
@@ -358,7 +357,6 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage)
         }
         first = false;
 
-
         if (!imageData.m_highlight.isValid() || ColorUtils::chroma(color) > ColorUtils::chroma(imageData.m_highlight)) {
             imageData.m_highlight = color;
         }
@@ -377,41 +375,55 @@ ImageData ImageColors::generatePalette(const QImage &sourceImage)
 
 QVariantList ImageColors::palette() const
 {
-    if (m_futureImageData) qWarning()<< m_futureImageData->future().isFinished();
-    return_fallback(m_fallbackPalette)
-    return m_imageData.m_palette;
+    if (m_futureImageData) {
+        qCWarning(KirigamiLog) << m_futureImageData->future().isFinished();
+    }
+    return_fallback(m_fallbackPalette) return m_imageData.m_palette;
 }
 
 ColorUtils::Brightness ImageColors::paletteBrightness() const
 {
+    /* clang-format off */
     return_fallback(m_fallbackPaletteBrightness)
-    return qGray(m_imageData.m_dominant.rgb()) < 128
-            ? ColorUtils::Dark
-            : ColorUtils::Light;
+
+    return qGray(m_imageData.m_dominant.rgb()) < 128 ? ColorUtils::Dark : ColorUtils::Light;
+    /* clang-format on */
 }
 
 QColor ImageColors::average() const
 {
+    /* clang-format off */
     return_fallback_finally(m_fallbackAverage, linkBackgroundColor)
+
     return m_imageData.m_average;
+    /* clang-format on */
 }
 
 QColor ImageColors::dominant() const
 {
+    /* clang-format off */
     return_fallback_finally(m_fallbackDominant, linkBackgroundColor)
+
     return m_imageData.m_dominant;
+    /* clang-format on */
 }
 
 QColor ImageColors::dominantContrast() const
 {
+    /* clang-format off */
     return_fallback_finally(m_fallbackDominantContrasting, linkBackgroundColor)
+
     return m_imageData.m_dominantContrast;
+    /* clang-format on */
 }
 
 QColor ImageColors::foreground() const
 {
+    /* clang-format off */
     return_fallback_finally(m_fallbackForeground, textColor)
-    if (paletteBrightness() == ColorUtils::Dark) {
+
+    if (paletteBrightness() == ColorUtils::Dark)
+    {
         if (qGray(m_imageData.m_closestToWhite.rgb()) < 200) {
             return QColor(230, 230, 230);
         }
@@ -422,11 +434,14 @@ QColor ImageColors::foreground() const
         }
         return m_imageData.m_closestToBlack;
     }
+    /* clang-format on */
 }
 
 QColor ImageColors::background() const
 {
+    /* clang-format off */
     return_fallback_finally(m_fallbackBackground, backgroundColor)
+
     if (paletteBrightness() == ColorUtils::Dark) {
         if (qGray(m_imageData.m_closestToBlack.rgb()) > 80) {
             return QColor(20, 20, 20);
@@ -438,29 +453,38 @@ QColor ImageColors::background() const
         }
         return m_imageData.m_closestToWhite;
     }
+    /* clang-format on */
 }
 
 QColor ImageColors::highlight() const
 {
+    /* clang-format off */
     return_fallback_finally(m_fallbackHighlight, linkColor)
+
     return m_imageData.m_highlight;
+    /* clang-format on */
 }
 
 QColor ImageColors::closestToWhite() const
 {
+    /* clang-format off */
     return_fallback(Qt::white)
     if (qGray(m_imageData.m_closestToWhite.rgb()) < 200) {
         return QColor(230, 230, 230);
     }
+    /* clang-format on */
+
     return m_imageData.m_closestToWhite;
 }
 
 QColor ImageColors::closestToBlack() const
 {
+    /* clang-format off */
     return_fallback(Qt::black)
     if (qGray(m_imageData.m_closestToBlack.rgb()) > 80) {
         return QColor(20, 20, 20);
     }
+    /* clang-format on */
     return m_imageData.m_closestToBlack;
 }
 
